/* $USAGI: ip6packettools.c,v 1.2 2001/12/18 18:33:38 yoshfuji Exp $ */

/*
 * IPSECv6 code
 * Copyright (C) 2000 Stefan Schlott
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: ip6packettools.c,v 1.4 2001/06/14 09:33:17 mk Exp $
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/skbuff.h>
#include <net/ipv6.h>
#include <linux/inet.h>
#include <linux/ip6packettools.h>
#include <linux/sysctl.h>
#include <linux/ipsec.h>
#include <linux/ipsec6.h>

void analyze_ip6_packet(char* packet, struct ip6_packet_refs *result) 
{
	unsigned char *hdrstart;
	__u8 nexthdr;

	if (!result) return;
	memset(result,0,sizeof(struct ip6_packet_refs));
	result->resttype=NEXTHDR_NONE;
	result->ipv6hdr=(struct ipv6hdr *)packet;	
	hdrstart = packet + sizeof(struct ipv6hdr);
	nexthdr = result->ipv6hdr->nexthdr;
	while (hdrstart!=NULL)
	{
		switch (nexthdr) {
			/* Headers having a nexthdr */
			case NEXTHDR_ROUTING:
				result->rthdr=(struct ipv6_rt_hdr*)hdrstart;
				nexthdr = result->rthdr->nexthdr;
				hdrstart += (result->rthdr->hdrlen+1)*8;
				break;
			case NEXTHDR_HOP:
				result->hopopthdr=(struct ipv6_hopopt_hdr*)hdrstart;
				nexthdr = result->hopopthdr->nexthdr;
				hdrstart+=(result->hopopthdr->hdrlen+1)*8;
				break;
			case NEXTHDR_DEST:
				result->destopthdr=(struct ipv6_destopt_hdr*)hdrstart;
				nexthdr = result->destopthdr->nexthdr;
				hdrstart+=(result->destopthdr->hdrlen+1)*8;
				break;
			case NEXTHDR_AUTH:
				result->authhdr=(struct ipv6_auth_hdr*)hdrstart;
				nexthdr = result->authhdr->nexthdr;
				hdrstart+=(result->authhdr->hdrlen+2)*4;
				break;
			/* Trailing headers */
			case NEXTHDR_NONE:
				result->rest=NULL;
				result->resttype=nexthdr;
				hdrstart=NULL;
				break;
			case NEXTHDR_ESP:
			case NEXTHDR_IPV6:
			case NEXTHDR_FRAGMENT:
			case NEXTHDR_ICMP:
			case NEXTHDR_TCP:
			case NEXTHDR_UDP:
				result->rest=hdrstart;
				result->resttype=nexthdr;
				hdrstart=NULL;
				break;
			/* Default: Unknown header */
			default:
				hdrstart=NULL;
				break;
		}
	}
}

void print_ip6packet(struct ip6_packet_refs refs)
{
	char srcadr[50], dstadr[50], buf[1024], *s;
	int length;

	length = ntohs(refs.ipv6hdr->payload_len) + sizeof(struct ipv6hdr);

	in6_ntop(&refs.ipv6hdr->saddr, srcadr);
	in6_ntop(&refs.ipv6hdr->daddr, dstadr);
	sprintf(buf,"src: %s dest: %s total_length: %u",
		srcadr,dstadr,length);
	IPSEC6_DEBUG("IPv6 header (%s)\n",buf);
	if (refs.rthdr) {
		IPSEC6_DEBUG(" Routing header\n");
	}
	if (refs.destopthdr) {
		IPSEC6_DEBUG(" Destination options header\n");
	}
	if (refs.hopopthdr) {
		IPSEC6_DEBUG(" Hop-by-hop options header\n");
	}
	if (refs.authhdr) {
		sprintf(buf,"nexthdr: %u len: %u, spi: 0x%x seq: 0x%x",
			refs.authhdr->nexthdr,refs.authhdr->hdrlen,
			ntohl(refs.authhdr->spi),ntohl(refs.authhdr->seq_no));
		IPSEC6_DEBUG(" Authentication header (%s)\n",buf);
	}
	switch (refs.resttype) {
		case NEXTHDR_FRAGMENT:
			IPSEC6_DEBUG(" Fragment header\n");
			break;
		case NEXTHDR_ICMP:
			switch (((struct icmp6hdr*)refs.rest)->icmp6_type) {
				case ICMPV6_DEST_UNREACH:
					s = "DEST_UNREACH"; break;
				case ICMPV6_PKT_TOOBIG:
					s = "PKT_TOOBIG"; break;
				case ICMPV6_TIME_EXCEED:
					s = "TIME_EXCEED"; break;
				case ICMPV6_PARAMPROB:
					s = "PARAMPROB"; break;
				case ICMPV6_ECHO_REQUEST:
					s = "ECHO_REQUEST"; break;
				case ICMPV6_ECHO_REPLY:
					s = "ECHO_REPLY"; break;
				case ICMPV6_MGM_QUERY:
					s = "MGM_QUERY"; break;
				case ICMPV6_MGM_REPORT:
					s = "MGM_REPORT"; break;
				case ICMPV6_MGM_REDUCTION:
					s = "MGM_REDUCTION"; break;
				default:
					s = "unknown";
			}
			sprintf(buf,"type: %u (0x%x) - %s",
				((struct icmp6hdr*)refs.rest)->icmp6_type,
				((struct icmp6hdr*)refs.rest)->icmp6_type,s);
			IPSEC6_DEBUG(" ICMP header (%s)\n",buf);
			break;
		case NEXTHDR_UDP/*17*/:
			IPSEC6_DEBUG(" UDP packet\n");
			break;
		case NEXTHDR_TCP/*6*/:
			IPSEC6_DEBUG(" TCP packet\n");
			break;
		case NEXTHDR_ESP:
			sprintf(buf,"spi: %u seq: %u",
				ntohl(((struct ipv6_esp_hdr*)refs.rest)->spi),
				ntohl(((struct ipv6_esp_hdr*)refs.rest)->seq_no));
			IPSEC6_DEBUG(" Encrypted security payload (%s)\n",buf);
			break;
		default:
			IPSEC6_DEBUG(" Unknown header type %u (0x%x)\n",refs.resttype,
				refs.resttype);
	}
}

void print_dump(__u8* data, int length, int maxlines) 
{
	char buf[5*3*17],*bufend;
	int i,line;
	unsigned int x;
	
	bufend=buf;
	buf[0]=0;
	line=0;
	for (i=0; i<length; i++) {
		x=(unsigned int)data[i];
		sprintf(bufend,"%02x ",x);
		bufend+=3;
		if ((i+1)%8==0) {
			sprintf(bufend," ");
			bufend++;
		}
		if (((i+1)%16==0) || (i==length-1)) {
			line++;
			if (maxlines>0 && line==maxlines && i<length-1) 
				sprintf(bufend,"(...)");
			IPSEC6_DEBUG("%s\n",buf);
			bufend=buf;
			buf[0]=0;
			if (maxlines>0 && line==maxlines && i<length-1) 
				break;
		}
	}
	IPSEC6_DEBUG("%i bytes.\n",length);
}



